'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = Stream;

var _createPropStream = require('./utils/createPropStream');

var _createPropStream2 = _interopRequireDefault(_createPropStream);

var _observeStore = require('./utils/observeStore');

var _observeStore2 = _interopRequireDefault(_observeStore);

var _define = require('./utils/define');

var define = _interopRequireWildcard(_define);

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) newObj[key] = obj[key]; } } newObj.default = obj; return newObj; } }

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var should = require('chai').should();

/**
 * Creates a streaming context that updates a target context that
 * can be attached to a store and streamed to one or more contexts
 */
function Stream(streamKey, storeSelector) {
  var sharedContext = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : {};

  var _ref = arguments.length > 3 && arguments[3] !== undefined ? arguments[3] : {},
      _ref$verbose = _ref.verbose,
      verbose = _ref$verbose === undefined ? false : _ref$verbose;

  should.exist(streamKey, 'must specify a key to stream to context');
  should.exist(storeSelector, 'must specify a storeSelector factory function');

  var opts = { verbose: verbose };

  var propStream = (0, _createPropStream2.default)(streamKey, sharedContext, opts);

  var isAttached = false;
  function attach(store) {
    isAttached.should.be.false;
    isAttached = true;

    var dispose = propStream.listen();
    var select = storeSelector(store);
    should.exist(select);
    select.should.be.instanceof(Function);

    var unsubscribe = (0, _observeStore2.default)(store, select, propStream.set, opts);

    function detach() {
      unsubscribe();
      dispose();
      isAttached = false;
    }
    return detach;
  }

  return { attach: attach,
    context: propStream.context,
    get current() {
      return propStream.current();
    }
  };
}