'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _extends2 = require('babel-runtime/helpers/extends');

var _extends3 = _interopRequireDefault(_extends2);

var _freeze = require('babel-runtime/core-js/object/freeze');

var _freeze2 = _interopRequireDefault(_freeze);

exports.default = createPropStream;

var _eventemitter = require('eventemitter3');

var _eventemitter2 = _interopRequireDefault(_eventemitter);

var _define = require('./define');

var define = _interopRequireWildcard(_define);

var _symbols = require('./symbols');

var symbols = _interopRequireWildcard(_symbols);

var _metatypes = require('./metatypes');

var _metatypes2 = _interopRequireDefault(_metatypes);

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) newObj[key] = obj[key]; } } newObj.default = obj; return newObj; } }

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var should = require('chai').should();

/**
 * Injects a private streamed value as a getter only property of one or more contexts.
 * @param  {string} publicKey     The getter property name of the target context.
 * @param  {Object} sharedContext An optional shared context to sync (useful for globals and debugging).
 * @return {Object}               An object with controls for attaching and streaming to contexts.
 */
function createPropStream(publicKey) {
  var sharedContext = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};
  var opts = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : {};

  var _value = void 0;
  var metaType = (0, _metatypes2.default)('propStream', publicKey);

  var _contexts = [sharedContext];

  var EE = new _eventemitter2.default();
  var flags = (0, _freeze2.default)({ get listening() {
      return EE.listeners(symbols.set).length > 0;
    } });

  function set(value) {
    should.exist(value, 'value must exist to set in stream');
    EE.emit(symbols.set, value);
    return value;
  }

  function listener(value) {
    if (_contexts.some(function (x) {
      return x === undefined || x === null;
    })) console.warn('Context does not exist, you may not be cleaning up resources');
    should.exist(value, 'incoming prop stream value must exist');
    _value = value;
  }

  function listen() {
    EE.listeners(symbols.set).length.should.equal(0, 'Max of one listener allowed per prop stream');
    EE.on(symbols.set, listener);
    return dispose;
  }

  function dispose() {
    EE.listeners(symbols.set).length.should.equal(1, 'No listener running on prop stream. Did you forget to call listen?');
    EE.removeListener(symbols.set, listener);
    return listen;
  }

  function context(_context) {
    var requireValue = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : false;

    if (requireValue) should.exist(_value, 'value must be set prior to stream creation');
    define.strictProperty(_context, publicKey, {
      get: function get() {
        return _value;
      }
    }).should.be.true;

    define.symbolProperties(_context, metaType('context', _contexts.length));
    _contexts.push(_context);
    return _context;
  }

  return (0, _freeze2.default)((0, _extends3.default)({}, metaType.typedProps, { set: set,
    context: context,
    listen: listen,
    dispose: dispose,
    flags: flags,
    sharedContext: context(sharedContext),
    get contexts() {
      return _contexts;
    },
    current: function current() {
      if (_value === undefined) console.warn('prop stream must have a value set prior to polling current value');
      return _value;
    }
  }));
}