'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _stringify = require('babel-runtime/core-js/json/stringify');

var _stringify2 = _interopRequireDefault(_stringify);

exports.default = observeStore;

var _define = require('./define');

var define = _interopRequireWildcard(_define);

var _stats = require('./stats');

var _stats2 = _interopRequireDefault(_stats);

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) newObj[key] = obj[key]; } } newObj.default = obj; return newObj; } }

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var should = require('chai').should();

/** Observes redux store and pushes changes triggering a different state selection value */
function observeStore(store, select, onChange) {
  var opts = arguments.length > 3 && arguments[3] !== undefined ? arguments[3] : {};

  should.exist(store);
  should.exist(select);
  should.exist(onChange);
  select.should.be.instanceof(Function);
  onChange.should.be.instanceof(Function);
  var currentState = void 0;

  var counts = {};
  var resetCounts = function resetCounts() {
    counts.observed = 0;
    counts.bypassed = 0;
  };
  var observe = function observe() {
    counts.observed++;
  };
  var bypass = function bypass() {
    counts.bypassed++;
  };
  if (opts.verbose) resetCounts();

  function handleChange() {
    if (opts.verbose) observe();
    var nextState = select(store.getState());
    if (nextState === currentState) return opts.verbose ? bypass() : null;

    currentState = nextState;
    if (opts.verbose) {
      (0, _stats2.default)('react-redux-stream: ' + (0, _stringify2.default)(counts), currentState);
      resetCounts();
    }
    onChange(currentState);
  }

  var unsubscribe = store.subscribe(handleChange);
  handleChange();
  return unsubscribe;
}